import {AlignX, AlignY, Point, Rect, Size} from "./engine/datatypes.ts";
import {D} from "./engine/public.ts";
import {BG_INSET, FG_BOLD, FG_TEXT} from "./colors.ts";
import {DrawPile} from "./drawpile.ts";

type Button = {
  label: string,
  cbClick: () => void,
}

export class Hotbar {
  #drawpile: DrawPile;
  constructor() {
    this.#drawpile = new DrawPile();
  }

  get #cellSize(): Size {
    return new Size(96, 32);
}

  get size(): Size {
    let {w: cellW, h: cellH} = this.#cellSize;
    let w = this.#computeButtons().length * cellW;
    return new Size(w, cellH)
  }

  #computeButtons(): Button[] {
    let buttons: Button[] = [];
    buttons.push({
      label: "Skills",
      cbClick: () => alert("beep"),
    })
    /*
    buttons.push({
      label: "Thralls"
    })
     */
    return buttons;
  }

  update() {
    this.#drawpile.clear();

    let buttons = this.#computeButtons();

    let cellSize = this.#cellSize;

    let x = 0;
    let padding = 2;
    for (let b of buttons.values()) {
      let topLeft = new Point(x, 0);
      let topLeftPadded = topLeft.offset(new Point(padding, padding));
      let sizePadded = new Size(cellSize.w - padding * 2, cellSize.h - padding * 2);
      let center = topLeft.offset(new Point(cellSize.w / 2, cellSize.h / 2));

      this.#drawpile.addClickable(
        0,
        (hover) => {
          let [bg, fg, fgLabel] = [BG_INSET, FG_TEXT, FG_BOLD];
          if (hover) {
            [bg, fg, fgLabel] = [FG_TEXT, BG_INSET, BG_INSET];
          }
          D.fillRect(
            topLeftPadded.offset(new Point(-1, -1)),
            sizePadded.add(new Size(2, 2)),
            bg
          );
          D.drawRect(topLeftPadded, sizePadded, fg);
          D.drawText(b.label, center, fgLabel, {
            alignX: AlignX.Center,
            alignY: AlignY.Middle,
          })
        },
        new Rect(topLeftPadded, sizePadded),
        true,
        b.cbClick,
      );
      x += cellSize.w;
    }
    this.#drawpile.executeOnClick();

  }

  draw() {
    // D.fillRect(new Point(-4, -4), this.size.add(new Size(8, 8)), BG_INSET);
    this.#drawpile.draw();
  }
}

let active = new Hotbar();
export function getHotbar(): Hotbar {
  return active;
}